%%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%% Extract gait events (GE) from marker-based data
%%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% This work is part of the MOBILISE-D Project (https://www.mobilise-d.eu/)
%
% Author code:  Dr Tecla Bonci
%               email: t.bonci@sheffield.ac.uk
%
%%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%%
% This main script extracts the timing (in frames) of both left and right
% gait events. Exemplary data is provided.
% Please change the parameters listed below as relevant to the data you
% will be using. A description of these parameters is reported below.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clearvars
close all
clc

addpath('Functions');

%% load example data
load data

%% EXAMPLE DATA - DESCRIPTION
%  data is a structure where the 3D marker trajectories acquired in the
%  different tests/trials are organised as described in the "READ ME"
%  document. For each Test and Trial, the raw 3D trajectories are organised
%  in the field "headerTraj". The 3D trajectories of each marker are
%  arranged in a [n x 3] matrix, where the 3rd component represents the
%  vertical one.
%
%  PELVIC MARKER PLACEMENT:
%  The marker on the pelvis are called "DYNA0","DYNAREF","DYNAX",and
%  "DYNAY", as described in the article: Mazzà et al. (2021)
%  BMJ Open 2021 Dec 2;11(12):e050785
%  doi: 10.1136/bmjopen-2021-050785.
%
%  NOTE: if the data have been collected with a protocol following the
%  lower-body Plug-in-Gait, based on the Newington-Helen Hayes gait model
%  (Kadaba, M.P., Ramakrishnan, H.K., and Wootten, M.E. (1990): 
%  J Orthop Res. 1990 May;8(3):383-92. doi: 10.1002/jor.1100080310;
%  Davis III, R.B., Ounpuu, S., Tyburski, D. and Gage, J.R., (1991)
%  Human movement science, 10(5), pp.575-587), the algorithm automatically 
%  uses the markers placed on both ASIS and PSIS.
%%

%% PARAMETERS TO SET - change these as relevant
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% convertoFromMillimetersToMeters   Logic input: TRUE/FALSE;
%                                   The script expects marker trajectories 
%                                   in m, if you need to convert your data
%                                   from mm to m, the variable
%                                   "convertoFromMillimetersToMeters" has
%                                   to be set to "TRUE",
%                                   "FALSE" otherwise (position data
%                                   already in m). Convert you data,
%                                   otherwise.
% 
%                      headerTraj   Name of the field within the data
%                                   structure where the different marker
%                                   trajectories are organised.
%
%                           sides   This is a [1x2] cell where the
%                                   left/right sides are indicated.
%                                   Examples:
%                                   sides = {'R','L'};
%                                   sides = {'r_','l_'};
%
%                         FootMrk   This is a [1x2] cell where the markers
%                                   used detect the initial(FootMrk{1}) and
%                                   final(FootMrk{2})contacts are indicated.
%                                   Please notice that the algorithm has
%                                   been tested for markers placed on heel
%                                   and toe locations.
%
%                              fs   Sampling frequency [Hz].
%
%                             fig   Logic input: TRUE/FALSE;
%                                   The overall gait event identification
%                                   figure is shown.
%
%                 MobiliseDformat   Logic input: TRUE/FALSE;
%                                   Indicate in which format the data is
%                                   provided. If the data is not provided
%                                   in the Mobilise-D format, the data has
%                                   to be organised in a struct where the
%                                   sample frequency "Fs" is indicated. The
%                                   raw 3D marker trajectories are arranged
%                                   in a [n x 3] matrix, where the 3rd
%                                   component represents the vertical one,
%                                   and included in the "headerTraj" field.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

convertoFromMillimetersToMeters = true; 
headerTraj = 'Mrks';
sides = {'R','L'};
FootMrk = {'HEEL','TOE'};
fig = true;
MobiliseDformat = true;
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if MobiliseDformat
    TM = fieldnames(data);
    for tm = 1:size(TM,1)
        Tests = fieldnames(data.(TM{tm}));

        for t = 1:size(Tests,1)
            Trials = fieldnames(data.(TM{tm}).(Tests{t}));

            for tr = 1:size(Trials,1)     

                dataNow = data.(TM{tm}).(Tests{t}).(Trials{tr}).Standards.Stereophoto_raw;

                %% Gaps identification and Filtering
                [dataNow, MrkGaps, AnatomicalAxesNotDef] = dataFiltering(dataNow, headerTraj, convertoFromMillimetersToMeters);
                fs = dataNow.Fs;
                %% GE detection
                markers = fieldnames(dataNow.(headerTraj)); mrkPelvis = markers(contains(markers, 'DYN'));
                for s = 1:size(sides,2)
                    [GE.(TM{tm}).(Tests{t}).(Trials{tr}).IC.(sides{s}), GE.(TM{tm}).(Tests{t}).(Trials{tr}).FC.(sides{s})] = ...
                        detect_GE_Zeni_Enh_Ghoussayni(dataNow, headerTraj, FootMrk, (sides{s}), fs, mrkPelvis, fig, AnatomicalAxesNotDef, MrkGaps);
                end %L/R
            end %Trials
        end %Test
    end %TimeMeasure
    
else
    % data does not follow the Mobilise-D format
    dataNow = data.TimeMeasure1.Test10.Trial1.Standards.Stereophoto_raw;
    
    %% Gaps identification and Filtering
    [dataNow, MrkGaps, AnatomicalAxesNotDef] = dataFiltering(dataNow, headerTraj, convertoFromMillimetersToMeters);
    fs = dataNow.Fs;
    
    %% GE detection
    markers = fieldnames(dataNow.(headerTraj)); mrkPelvis = markers(contains(markers, 'DYN'));
    for s = 1:size(sides,2)
        [GE.IC.(sides{s}), GE.FC.(sides{s})] = ...
            detect_GE_Zeni_Enh_Ghoussayni(dataNow, headerTraj, FootMrk, (sides{s}), fs, mrkPelvis, fig, AnatomicalAxesNotDef, MrkGaps);
    end %L/R
end